getDuplexBPfeat <- function(mirMap, tarMap, seed = 2:8) {
  
  mirWC  <- grepRaw(pattern = "\\|", mirMap, all = TRUE)
  mirGU  <- grepRaw(pattern = "\\!", mirMap, all = TRUE)
  mirBP  <- grepRaw(pattern = "\\!|\\|", mirMap, all = TRUE)
  mirIBP <- grepRaw(pattern = "\\]|\\[", mirMap, all = TRUE) 
  
  tarBP  <- grepRaw(pattern = "\\!|\\|", tarMap, all = TRUE)
  tarIBP <- grepRaw(pattern = "\\]|\\[", tarMap, all = TRUE)
  tarIBP <- tarIBP[tarIBP > min(tarBP) & tarIBP < max(tarBP)]
  
  outseed <- seq(max(seed) + 1, nchar(mirMap))
  
  mirMap.seed    <- substr(mirMap, min(seed), max(seed))
  mirMap.outseed <- substr(mirMap, min(outseed), max(outseed))
  
  splitCWC <- "\\!|\\.|\\]|\\["
  
  mirCWC <- strsplit(mirMap, split = splitCWC)[[1]]
  mirCWC <- mirCWC[mirCWC != ""]
  mirCWC.seed    <- strsplit(mirMap.seed, split = splitCWC)[[1]]
  mirCWC.outseed <- strsplit(mirMap.outseed, split = splitCWC)[[1]]
  
  splitCBP <- "\\.|\\]|\\["
  
  mirCBP <- strsplit(mirMap, split = splitCBP)[[1]]
  mirCBP <- mirCBP[mirCBP != ""]
  mirCBP.seed    <- strsplit(mirMap.seed, split = splitCBP)[[1]]
  mirCBP.outseed <- strsplit(mirMap.outseed, split = splitCBP)[[1]]
  
  tarCBP <- strsplit(tarMap, split = splitCBP)[[1]]
  tarCBP <- tarCBP[tarCBP != ""]
  
  duplexFeat <- c(
    duplex.WC = length(mirWC),
    duplex.GU = length(mirGU),
    duplex.BP = length(mirBP)
  )
  mirFeat <- c(
    mir.WC.seed    = sum(mirWC %in% seed),
    mir.GU.seed    = sum(mirGU %in% seed),
    mir.BP.seed    = sum(mirBP %in% seed),
    mir.WC.outseed = sum(mirWC %in% outseed),
    mir.GU.outseed = sum(mirGU %in% outseed),
    mir.BP.outseed = sum(mirBP %in% outseed),
    mir.IntramolBP = length(mirIBP) / 2
  )
  mirWCFeat <- c(
    mir.WC.1stpos  = min(mirWC),
    mir.WC.lastpos = max(mirWC),
    mir.WC.mindist = min(diff(mirWC)),
    mir.WC.avgdist = mean(diff(mirWC)),
    mir.WC.maxdist = max(diff(mirWC))
  )
  mirBPFeat <- c(
    mir.BP.1stpos  = min(mirBP),
    mir.BP.lastpos = max(mirBP), 
    mir.BP.mindist = min(diff(mirWC)),
    mir.BP.avgdist = mean(diff(mirBP)),
    mir.BP.maxdist = max(diff(mirWC))
  )
  mirCWCFeat <- c(
    mir.CWC.numreg = length(mirCWC), 
    mir.CWC.minlen = min(nchar(mirCWC)),
    mir.CWC.avglen = mean(nchar(mirCWC)), 
    mir.CWC.maxlen = max(nchar(mirCWC)),
    mir.CWC.seed.maxlen    = max(nchar(mirCWC.seed)), 
    mir.CWC.outseed.maxlen = max(nchar(mirCWC.outseed))
  )
  mirCBPFeat <- c(
    mir.CBP.numreg = length(mirCBP), 
    mir.CBP.minlen = min(nchar(mirCBP)),
    mir.CBP.avglen = mean(nchar(mirCBP)), 
    mir.CBP.maxlen = max(nchar(mirCBP)),
    mir.CBP.seed.maxlen    = max(nchar(mirCBP.seed)), 
    mir.CBP.outseed.maxlen = max(nchar(mirCBP.outseed))
  )
  tarFeat <- c(
    tar.MRE.lenght = length(seq(min(tarBP), max(tarBP))),
    tar.BP.avgdist = mean(diff(tarBP)), 
    tar.BP.maxdist = max(diff(tarBP)), 
    tar.CBP.numreg = length(tarCBP),
    tar.CBP.minlen = min(nchar(tarCBP)), 
    tar.CBP.avglen = mean(nchar(tarCBP)), 
    tar.CBP.maxlen = max(nchar(tarCBP)), 
    tar.MRE.IntramolBP = length(tarIBP) / 2
  )
  
  output <- c(duplexFeat, mirFeat, mirWCFeat, mirBPFeat, 
              mirCWCFeat, mirCBPFeat, tarFeat)
  
  return(output)
}
