getDuplexSSfeat <- function(mirMap, tarMap) {
  
  mirBP <- grepRaw(pattern = "\\!|\\|", mirMap, all = T)
  tarBP <- grepRaw(pattern = "\\!|\\|", tarMap, all = T)
  
  mirInterBPdist <- diff(mirBP)
  tarInterBPdist <- diff(tarBP)
  
  mirInterBPhairpin <- substr(mirMap, start = min(mirBP), stop = max(mirBP))
  mirInterBPhairpin <- strsplit(mirInterBPhairpin, split = "\\!|\\|")[[1]][-1]
  mirInterBPhairpin <- gsub(pattern = "\\.", replacement = "", mirInterBPhairpin) 
  
  tarInterBPhairpin <- substr(tarMap, start = min(tarBP), stop = max(tarBP))
  tarInterBPhairpin <- strsplit(tarInterBPhairpin, split = "\\!|\\|")[[1]][-1]
  tarInterBPhairpin <- gsub(pattern = "\\.", replacement = "", tarInterBPhairpin)
  
  for (i in seq(along = mirInterBPhairpin)) {
    if (mirInterBPhairpin[i] == "") {
      mirInterBPhairpin[i] <- "0"
    } else {
      brackets <- strsplit(mirInterBPhairpin[i], "")[[1]]
      numArrgt <- rep(1, length(brackets))
      numArrgt[brackets == "]"] <- -1
      numArrgt <- cumsum(numArrgt)
      mirInterBPhairpin[i] <- sum(numArrgt == 0)
    }
  }
  
  for (i in seq(along = tarInterBPhairpin)) {
    if (tarInterBPhairpin[i] == "") {
      tarInterBPhairpin[i] <- "0"
    } else {
      brackets <- strsplit(tarInterBPhairpin[i], "")[[1]]
      numArrgt <- rep(1, length(brackets))
      numArrgt[brackets == "]"] <- -1
      numArrgt <- cumsum(numArrgt)
      tarInterBPhairpin[i] <- sum(numArrgt == 0)
    }
  }
  
  mirInterBPhairpin <- as.numeric(mirInterBPhairpin)
  tarInterBPhairpin <- as.numeric(tarInterBPhairpin)
  
  mirBulge.cond <- c(
    mirInterBPdist >= 2 & tarInterBPdist == 1 & mirInterBPhairpin == 0
  )
  mirBulge.len <- mirInterBPdist[mirBulge.cond] - 1
  if (length(mirBulge.len) == 0) {
    mir.Bulge.num    <- 0
    mir.Bulge.minlen <- 0
    mir.Bulge.avglen <- 0
    mir.Bulge.maxlen <- 0
    mir.Bulge.sumlen <- 0
  } else {
    mir.Bulge.num    <- length(mirBulge.len)
    mir.Bulge.minlen <- min(mirBulge.len)
    mir.Bulge.avglen <- mean(mirBulge.len)
    mir.Bulge.maxlen <- max(mirBulge.len)
    mir.Bulge.sumlen <- sum(mirBulge.len)
  }
  tarBulge.cond <- c(
    tarInterBPdist >= 2 & mirInterBPdist == 1 & tarInterBPhairpin == 0
  )
  tarBulge.len <- tarInterBPdist[tarBulge.cond] - 1
  if (length(tarBulge.len) == 0) {
    tar.Bulge.num    <- 0
    tar.Bulge.minlen <- 0
    tar.Bulge.avglen <- 0
    tar.Bulge.maxlen <- 0
    tar.Bulge.sumlen <- 0
  } else {
    tar.Bulge.num    <- length(tarBulge.len)
    tar.Bulge.minlen <- min(tarBulge.len)
    tar.Bulge.avglen <- mean(tarBulge.len)
    tar.Bulge.maxlen <- max(tarBulge.len)
    tar.Bulge.sumlen <- sum(tarBulge.len)
  }
  symLoop.cond <- c(
    mirInterBPdist >= 2 & mirInterBPdist == tarInterBPdist &
      mirInterBPhairpin == 0 & tarInterBPhairpin == 0
  )
  symLoop.len <- mirInterBPdist[symLoop.cond] - 1 
  if (length(symLoop.len) == 0) {
    duplex.SymLoop.num <- 0
    duplex.SymLoop.minlen <- 0
    duplex.SymLoop.avglen <- 0
    duplex.SymLoop.maxlen <- 0
    duplex.SymLoop.sumlen <- 0
  } else {
    duplex.SymLoop.num <- length(symLoop.len)
    duplex.SymLoop.minlen <- min(symLoop.len)
    duplex.SymLoop.avglen <- mean(symLoop.len)
    duplex.SymLoop.maxlen <- max(symLoop.len)
    duplex.SymLoop.sumlen <- sum(symLoop.len)
  }
  asymLoop.cond <- c(
    mirInterBPdist >= 2 & tarInterBPdist >= 2 &
      mirInterBPhairpin == 0 & tarInterBPhairpin == 0
  )
  asymLoop.mirlen <- mirInterBPdist[asymLoop.cond] - 1
  asymLoop.tarlen <- tarInterBPdist[asymLoop.cond] - 1
  if (length(asymLoop.mirlen) == 0) {
    duplex.AsymLoop.num <- 0
    mir.AsymLoop.minlen <- 0
    mir.AsymLoop.avglen <- 0
    mir.AsymLoop.maxlen <- 0
    mir.AsymLoop.sumlen <- 0
    tar.AsymLoop.minlen <- 0
    tar.AsymLoop.avglen <- 0
    tar.AsymLoop.maxlen <- 0
    tar.AsymLoop.sumlen <- 0
  } else {
    duplex.AsymLoop.num <- length(asymLoop.mirlen)
    mir.AsymLoop.minlen <- min(asymLoop.mirlen)
    mir.AsymLoop.avglen <- mean(asymLoop.mirlen)
    mir.AsymLoop.maxlen <- max(asymLoop.mirlen)
    mir.AsymLoop.sumlen <- sum(asymLoop.mirlen)
    tar.AsymLoop.minlen <- min(asymLoop.tarlen)
    tar.AsymLoop.avglen <- mean(asymLoop.tarlen)
    tar.AsymLoop.maxlen <- max(asymLoop.tarlen)
    tar.AsymLoop.sumlen <- sum(asymLoop.tarlen)
  } 
  mir3WJ.cond <- c(
    (mirInterBPdist >= 2 | tarInterBPdist >= 2) & 
      mirInterBPhairpin == 1 & tarInterBPhairpin == 0
  )
  mir.3WJ.num <- sum(mir3WJ.cond)
  tar3WJ.cond <- c(
    (mirInterBPdist >= 2 | tarInterBPdist >= 2) & 
      mirInterBPhairpin == 0 & tarInterBPhairpin == 1
  )
  tar.3WJ.num <- sum(tar3WJ.cond)
  multiWJ.cond <- c(
    (mirInterBPdist >= 2 | tarInterBPdist >= 2) & 
      (mirInterBPhairpin + tarInterBPhairpin) > 1
  )
  duplex.multiWJ.num <- sum(multiWJ.cond)
  
  output <- c(
    mir.Bulge.num, mir.Bulge.minlen, mir.Bulge.avglen, 
    mir.Bulge.maxlen, mir.Bulge.sumlen, 
    tar.Bulge.num, tar.Bulge.minlen, tar.Bulge.avglen, 
    tar.Bulge.maxlen, tar.Bulge.sumlen, 
    duplex.SymLoop.num, 
    duplex.SymLoop.minlen, duplex.SymLoop.avglen, 
    duplex.SymLoop.maxlen, duplex.SymLoop.sumlen,
    duplex.AsymLoop.num, 
    mir.AsymLoop.minlen, mir.AsymLoop.avglen, 
    mir.AsymLoop.maxlen, mir.AsymLoop.sumlen, 
    tar.AsymLoop.minlen, tar.AsymLoop.avglen, 
    tar.AsymLoop.maxlen, tar.AsymLoop.sumlen,
    mir.3WJ.num, tar.3WJ.num, duplex.multiWJ.num
  )
  names(output) <- c(
    "mir.Bulge.num", "mir.Bulge.minlen", "mir.Bulge.avglen", 
    "mir.Bulge.maxlen", "mir.Bulge.sumlen", 
    "tar.Bulge.num", "tar.Bulge.minlen", "tar.Bulge.avglen", 
    "tar.Bulge.maxlen","tar.Bulge.sumlen", 
    "duplex.SymLoop.num", 
    "duplex.SymLoop.minlen", "duplex.SymLoop.avglen", 
    "duplex.SymLoop.maxlen", "duplex.SymLoop.sumlen", 
    "duplex.AsymLoop.num", 
    "mir.AsymLoop.minlen", "mir.AsymLoop.avglen", 
    "mir.AsymLoop.maxlen", "mir.AsymLoop.sumlen", 
    "tar.AsymLoop.minlen", "tar.AsymLoop.avglen", 
    "tar.AsymLoop.maxlen", "tar.AsymLoop.sumlen", 
    "mir.3WJ.num", "tar.3WJ.num", "duplex.multiWJ.num"
  )
  
  return(output)
}

