\name{getDuplexSSfeat}
\alias{getDuplexSSfeat}
\title{
Get Duplex Secondary Structure Features
}
\description{
Performs a count of different structural motifs and quantifies their size.
}
\usage{
getDuplexSSfeat(mirMap, tarMap)
}
\arguments{
  \item{mirMap}{
  character string containing the interaction map for microRNA.
  See \code{\link{getDuplexMap}} for more details of interaction maps.
}
  \item{tarMap}{
  character string containing the interaction map for target RNA.
}
}
\details{
The different structural motifs evaluated are:
miRNA bulge (mir.Bulge),
target bulge (tar.Bulge),
duplex symmetric loop (duplex.SymmLoop),
duplex asymmetric loop (duplex.AsymLoop),
three way junction with hairpin in the miRNA (mir.3WJ),
three way junction with hairpin in the target (tar.3WJ)
and duplex multi-way junction (duplex.multiWJ).

For bulges and loops the following measurements are
carried out:
number of motifs (.num),
length or number of nucleotides of the smallest
motif (.minlen),
length or number of nucleotides of the largest
motif (.maxlen),
average length of the motifs (.avglen),
cumulative length of all motifs of the same
class (.sumlen).
In the case of asymmetric loops,
these measurements are made on the miRNA
and on the target separately.
For the rest of the structures,
only the number motifs is recorded.
}
\value{
getDuplexSSfeat returns a named numeric vector
containing the measurements performed.
}
\note{
G-U wobbles are evaluated as base pairs.
}
\examples{
# Example of asymmetric loops
mir_map <- "..||||...|||||....||.."
tar_map <- "..||||.....|||||.....||.."
getDuplexSSfeat(mirMap = mir_map, tarMap = tar_map)

# Example of target three way junction
mir_map <- "..||||...|||||....."
tar_map <- "..||||...[[[[[[....]]]]]]...|||||....."
getDuplexSSfeat(mirMap = mir_map, tarMap = tar_map)

# Example of duplex multi way junction
mir_map <- "..||||..[[[...]]]..|||||....."
tar_map <- "..||||...[[[[[[....]]]]]]...|||||....."
getDuplexSSfeat(mirMap = mir_map, tarMap = tar_map)

# Another example of multi way junction
mir_map <- "..||||...|||||....."
tar_map <- "..||||...[[[[[[....]]]]]]...[[[....]]]...|||||....."
getDuplexSSfeat(mirMap = mir_map, tarMap = tar_map)
}
