\name{getInteractType}
\alias{getInteractType}
\title{
Identification of MiRNA-Target Interaction Type
}
\description{
Can perform different classifications of the interactions
depending on the parameters of the function.
}
\usage{
getInteractType(mirMap, tarMap, tarSeq, supp = TRUE, mism = TRUE,
                suppNt = 4, compNt = 5, scReg = NULL)
}
\arguments{
  \item{mirMap}{
  character string containing the interaction map for microRNA.
  See \code{\link{getDuplexMap}} for more details of interaction map.
}
  \item{tarMap}{
  character string containing the interaction map for target RNA.
}
  \item{tarSeq}{
  character string containing the nucleotide sequence of target RNA.
}
  \item{supp}{
  logical value indicating whether a distinction should be made between
  canonical sites and canonical sites with 3' supplementary union.
}
  \item{mism}{
  logical value indicating whether to count the number of mismatches
  in non-canonical sites.
}
  \item{suppNt}{
  integer indicating the number of consecutive base pairs to consider
  that there is a 3' supplementary site.
}
\item{compNt}{
  integer indicating the number of consecutive base pairs to consider
  that there is a 3' complementary site; only when there is a mismatch
  in the seed region.
}
\item{scReg}{
  vector of integers indicating the region of the microRNA where
  to evaluate the existence of supplementary or complementary interactions.
  If NULL scrReg = 9:nchar(mirMap).
}
}
\details{

}
\value{
Possible outputs of the function are:

  \item{offset_6mer}{
  Watson-Crick match from nucleotides 3–8 of the miRNA.
}
  \item{6mer}{
  Watson-Crick match from nucleotides 2–7 of the miRNA.
}
  \item{7mer_m8}{
  Watson-Crick match from nucleotides 2–8 of the miRNA.
}
  \item{7mer_A1}{
  Watson-Crick match from nucleotides 2-7 of the miRNA
  and an adenine (A) at position 1.
}
  \item{8mer}{
  Watson-Crick match from nucleotides 2-8 of the miRNA
  and an adenine (A) at position 1.
}
  \item{NonCa_centred}{
  Watson-Crick match from nucleotides 4-15 or 5-16 of the miRNA.
}
  \item{NonCa_compensatory}{
  one mismatch in miRNA seed (nucleotides 2-7) and
  some consecutive base pairs in 3' region.
}
  \item{NonCa}{
  one or moore mismatches in miRNA seed (nucleotides 2-7) and
  no 3' compensatory binding.
}
  \item{not_defined}{
  at least two Watson-Crick base pairs are needed
  for the interaction to be defined.
}
  \item{no_binding}{
  there is no interaction.
}

If supp = TRUE, other possible outputs are:
offset_6mer_supp, 6mer_supp, 7mer_m8_supp,
7mer_A1_supp and 8mer_supp.

If msim = TRUE, ohter posible outputs are:
NonCa_1mism, ..., Nonca_6mism.

}
\examples{
mir_map <- ".|||||||.!||||!|..|||."
tar_map <- "...[[[.[[[[..[[.....]]..]]]]]]]..................|||||||....!||||!|......|||"
tar_seq <- "GCCUUUACCUGUUUAUUUUUUGCUGCUUUAUACCAAUCCCUUUUCCCUCUGAUUGUUUAAUAUUUUCAGCAGGCCA"

# Same interaction but different classifications
getInteracType(mirMap = mir_map, tarMap = tar_map, tarSeq = tar_seq, supp = TRUE)
getInteracType(mirMap = mir_map, tarMap = tar_map, tarSeq = tar_seq, supp = TRUE, suppNt = 5)
getInteracType(mirMap = mir_map, tarMap = tar_map, tarSeq = tar_seq, supp = FALSE)
}

